/*

 Addressable latch output example
 Controls 16 outputs, one at a time, using a CD4099 multiplexer.
 
 This sketch loops over the 16 channels of a CD4099 latch, switching
 the input to each output channel in turn.  It then fades up and down the 
 input channel using analogWrite().
 
 The circuit:
 * CD4099 multiplexer attached as follows:
 - address pin A0: digital I/O 2
 - address pin A1: digital I/O 3
 - address pin A2: digital I/O 4
 - data pin: digital I/O pin 6
 - reset pin: low
 - write disable pin: digital I/O pin 5
 - clear pin: digital I/O pin 7
 - LEDs attached from each of the CD4067's output channels
 to ground
 
 created 21 May 2009
 by Tom Igoe
 
 http://tigoe.net/pcomp/code/xxx for more
 
 */

// set up the pins.  put the address pin numbers in an array
// so they're easy to iterate over:
#include "WProgram.h"
void setup();
void loop();
void latchWrite(int whichChannel);
const int clearPin = 7;
const int writeEnablePin = 5;
const int outputPin  = 6;
const int channel[] = {
  4, 3, 2};

void setup() {
  // set all pins to output: 
  for (int thisPin = 2; thisPin < 8; thisPin++) {
    pinMode(thisPin, OUTPUT); 
  }
}
void loop() { 
  // enable latching by turning off the clear pin: 
  digitalWrite(clearPin, LOW);
  // loop over the channels, lighting each one: 
  for (int thisChannel = 0; thisChannel < 8; thisChannel++) {
    // disable the outputs: 
    digitalWrite(writeEnablePin, HIGH);
    // set the address: 
    latchWrite(thisChannel); 
    // set the desired state: 
    digitalWrite(outputPin, HIGH);
    // enable the output: 
    digitalWrite(writeEnablePin, LOW);
    delay(500); 
    Serial.begin(9600);
  }
  // disable latching by turning on the clear pin: 
  digitalWrite(clearPin, HIGH);
  // loop over the channels, lighting each one: 
  for (int thisChannel = 0; thisChannel < 8; thisChannel++) {
    // set the address: 
    latchWrite(thisChannel);
    Serial.println(thisChannel, BIN);
    // set the desired state: 
    digitalWrite(outputPin, HIGH);
    // enable the output: 
    digitalWrite(writeEnablePin, LOW);
    delay(500); 
  }
}

void latchWrite(int whichChannel) {
  // disable the outputs: 
 // digitalWrite(writeEnablePin, HIGH);
  // iterate over the number of pins you're using:
  for (int thisPin = 0; thisPin < 3; thisPin++) {
    // set the pin state based on the pin's bit value
    // in whichChannel:
    int pinState = bitRead(whichChannel, thisPin);
    digitalWrite(channel[thisPin],pinState);
  }
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

